
# Implement Game of Life!

Your goal for this exercise is to practice writing Lua bindings in C++.

In this exercise, you will create working Game of Life, that uses Lua scripts for the definition of the initial state of the game and for controlling the game itself.
This game is quite simple, please visit <https://en.wikipedia.org/wiki/Conway%27s_Game_of_Life> for more information.

You have to implement only the Lua bindings. GUI and game mechanics are implemented.

## Compiling

You need to have SDL2 GUI library installed, the project itself is standard cmake project:

```
mkdir -p build
cd build
cmake ..
make
```

## Running the code

Execute as:
```
./game_of_life test.lua
```

## Existing code

`window.hpp` contains `cell_state` and `game_window` types.
`cell_state` is an enum that represents the state the cells can be in - ALIVE or DEAD.
`game_window` is a GUI that you will use for visualization of the game.
This window can initialize itself and provides you with `game_window::cell_matrix` type.
You should use the `cell_matrix` type as storage for the actual state of the game.
If you want to visualize the `cell_matrix` just use method form the GUI window: `game_window::draw_cell_matrix`.

`main.cpp` contains two functions that represent the mechanics of the game.
`liveness` counts living cells around a specific cell in the matrix.
`gol_iterate` executes one iteration of GoL over the provided matrix and returns a new matrix.

So the idea is quite simple, you create game window, matrix and execute code that does something like this:
```C
while(true){
        cells = gol_iterate(cells);
        window.draw_cell_matrix(cells);
}
```

Except, that this will be done in Lua.

## What you should implement

You have GUI window to visualize the game and functions for game mechanics.
You have to fill in the lua bindings.

In file `test.lua` is a simple example file that initializes basic GoL game and executes it.
The binding expects that there is only one cell-matrix in the app.
For the Lua script, we use only numbers for setting cell state - 1 means alive, 0 means dead.

Your list of tasks for the bindings is to introduce following items into the Lua context, these are used by the script provided in this exercise:
 * variables `width` and `height` with actual dimensions of cell-matrix
        * hint: API uses term `global` for variables that are accessible from the top level of Lua scripts
 * function `set`
        * function takes three arguments as input - `x` and `y` coordinates and `state` of cell
        * this should set the state of the cell at the internal matrix at the coordinates
 * function `get`
        * function takes two arguments as input - `x` and `y`, it returns the state of the cell
        * this returns the state of the cell at appropriate coordinates from the internal matrix
 * function `gol_iterate`
        * function executes one iteration of GoL over the internal matrix
 * function `draw`
        * function draws actual state of cell-matrix in the GUI window
 * function `sleep`
        * function that allows the app to sleep for 100ms (tip: SDL_Delay)

You should also implement the part of the code that loads file provided as an argument to the binary and executes it.

## Do it correctly

You will find out that you need a global variable for accessing the window and cell-matrix from the functions.
This can be avoided, but I advise to do it after you managed to get the binding work with the global version.

Lua can bind the functions not as functions, but rather as closures.
You can give Lua variable when you bind the function, that will be accessible from the function itself.
The appropriate type here is called `lightuserdata`, which is `void *`.

This type gives you the ability to give Lua pointer to anything C/C++ related, which can be shuffled around as value within Lua.
With this, you can give each function pointers to your game state and avoid global variables.

To ease up the implementation of this, the list of relevant functions for this is:
`lua_pushlightuserdata`, `lua_pushclosure`, `lua_upvalueindex`, `lua_touserdata`

The keyword you are looking for in the manual is: upvalues

