#include <lauxlib.h>
#include <lualib.h>

#include <iostream>

#include "window.hpp"

using gol_window = game_window<64, 64, 8>;
using cell_matrix = typename gol_window::cell_matrix;

// Counts how many cells around `cells[x][y]` is alive
//
// WARNING: this function does not check boundaries of coordinates
int liveness(const cell_matrix& cells, std::size_t x, std::size_t y) {
	int res = 0;
	for (std::size_t i : {x - 1, x, x + 1}) {
		for (std::size_t j : {y - 1, y, y + 1}) {
			if (i == x && j == y) {
				continue;
			}
			if (cells[i][j] == cell_state::ALIVE) {
				res += 1;
			}
		}
	}
	return res;
}

// Does one step of Game of Live over inputa argument 
// and returns ne cell matrix
cell_matrix gol_iterate(const cell_matrix& cells) {
	cell_matrix res{cell_state::DEAD};
	for (std::size_t row_i = 1; row_i < cells.size() - 1; ++row_i) {
		const auto& row = cells[row_i];

		for (std::size_t col_i = 1; col_i < row.size() - 1; ++col_i) {
			int l = liveness(cells, row_i, col_i);
			const cell_state& c = row[col_i];
			if (c == cell_state::ALIVE && (l == 2 || l == 3)) {
				res[row_i][col_i] = cell_state::ALIVE;
			} else if (c == cell_state::DEAD && l == 3) {
				res[row_i][col_i] = cell_state::ALIVE;
			} else {
				res[row_i][col_i] = cell_state::DEAD;
			}
		}
	}
	return res;
}


int main(int argc, const char* argv[]) {
	if (SDL_Init(SDL_INIT_VIDEO) == -1) {
		std::cerr << "Failed to initialize SDL";
		return 1;
	}
	if (argc != 2) {
		std::cerr << "Provide filename for arg";
		return 1;
	}

	gol_window gui{};
	cell_matrix cells{cell_state::DEAD};

	lua_State* L = luaL_newstate();
	luaL_openlibs(L);

	

	SDL_Quit();
}
