# Try pybind11!

In this exercise you will try to implement a Python bindings for a really simple
graph library.

You can find the library in the file `graph.hpp`. The graph is represented by
adjacency list in each vector. The `Node` class is parametrized by type of data
associated with a node and an edge. The `Graph` class provides you with a method
for adding new vertices and it allows you to walk the graph in DFS manner. When
a vertex is visited or leaved, user provided callback is called.

## Compiling

The demo is compiled using CMake. Simply run the following command to compile it:
```
mkdir -p build
cd build
cmake ..
make -j8
```

Note that the `cmake ..` might take a while to complete since pybind11
dependency is downloaded (and FetchContent doesn't show progress bar).

## Running the demo

To execute the Python demo, just run:
```
PYTHONPATH=path_to_your_build_directory python3 example.py
```

## Your task

The goal is to basically make `example.py` executable by implementing python
binding for the library.

First, create Python classes `PlainGraph` and `PlainNode` mapping to `Graph<
std::string, Nothing >` and `Graph< std::string, Nothing >::NodeT` respectively.
The user should be able to create a graph, add nodes and add neighbors. Also, he
or she should be able to walks the graph supplying arbitrary Python function as
the callbacks.

Then try to implement Python classes `Graph` and `Node`. These should represent
a graph where vertex and edge data can be arbitrary Python values.

## Some tips

- when in doubt, look at the [pybind11
  documentation](https://pybind11.readthedocs.io/en/stable/)
- if your Python modules segfaults or causes double free, look for improper
  usage of `py::return_value_policy` (the automatic one does not fit everywhere)
